##############################################################################
# This software was developed by the University of Tennessee as part of the
# Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
# project funded by the US National Science Foundation.
#
# If you use DANSE applications to do scientific research that leads to
# publication, we ask that you acknowledge the use of the software with the
# following sentence:
#
# This work benefited from DANSE software developed under NSF award DMR-0520547
#
# Copyright 2008-2011, University of Tennessee
##############################################################################

""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY
            src\sans\models\include\GelFit.h
         AND RE-RUN THE GENERATOR SCRIPT
"""

from sans.models.BaseComponent import BaseComponent
from sans.models.sans_extension.c_models import CGelFitModel

def create_GelFitModel():
    """
       Create a model instance
    """
    obj = GelFitModel()
    # CGelFitModel.__init__(obj) is called by
    # the GelFitModel constructor
    return obj

class GelFitModel(CGelFitModel, BaseComponent):
    """ 
    Class that evaluates a GelFitModel model. 
    This file was auto-generated from src\sans\models\include\GelFit.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         lScale          = 3.5 
         gScale          = 1.7 
         zeta            = 16.0 
         radius          = 104.0 
         scale           = 2.0 
         background      = 0.01 

    """
        
    def __init__(self, multfactor=1):
        """ Initialization """
        self.__dict__ = {}
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        #apply(CGelFitModel.__init__, (self,)) 

        CGelFitModel.__init__(self)
        self.is_multifunc = False
		        
        ## Name of the model
        self.name = "GelFitModel"
        ## Model description
        self.description = """
        Structure factor for interacting particles:                   .
		Shibayama-Geissler Two-Length Scale Fit for Gels
		(GelFit)
		
		Sibayama; Tanaka; Han J Chem Phys(1992), 97(9), 6829-6841
		Mallam; Horkay; Hecht; Rennie; Geissler, Macromol(1991), 24, 543
        """
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['lScale'] = ['', None, None]
        self.details['gScale'] = ['', None, None]
        self.details['zeta'] = ['', None, None]
        self.details['radius'] = ['', None, None]
        self.details['scale'] = ['', None, None]
        self.details['background'] = ['', None, None]

        ## fittable parameters
        self.fixed = []
        
        ## non-fittable parameters
        self.non_fittable = []
        
        ## parameters with orientation
        self.orientation_params = []

        ## parameters with magnetism
        self.magnetic_params = []

        self.category = None
        self.multiplicity_info = None
        
    def __setstate__(self, state):
        """
        restore the state of a model from pickle
        """
        self.__dict__, self.params, self.dispersion = state
        
    def __reduce_ex__(self, proto):
        """
        Overwrite the __reduce_ex__ of PyTypeObject *type call in the init of 
        c model.
        """
        state = (self.__dict__, self.params, self.dispersion)
        return (create_GelFitModel, tuple(), state, None, None)
        
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(GelFitModel())   
       	
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        return CGelFitModel.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        return CGelFitModel.runXY(self, x)
        
    def evalDistribution(self, x):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CGelFitModel.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CGelFitModel.calculate_ER(self)
        
    def calculate_VR(self):
        """ 
        Calculate the volf ratio for P(q)*S(q)
        
        :return: the value of the volf ratio
        
        """       
        return CGelFitModel.calculate_VR(self)
              
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CGelFitModel.set_dispersion(self,
               parameter, dispersion.cdisp)
        
   
# End of file

