##############################################################################
# This software was developed by the University of Tennessee as part of the
# Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
# project funded by the US National Science Foundation.
#
# If you use DANSE applications to do scientific research that leads to
# publication, we ask that you acknowledge the use of the software with the
# following sentence:
#
# This work benefited from DANSE software developed under NSF award DMR-0520547
#
# Copyright 2008-2011, University of Tennessee
##############################################################################

""" 
Provide functionality for a C extension model

:WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
         DO NOT MODIFY THIS FILE, MODIFY
            src\sans\models\include\starpolymer.h
         AND RE-RUN THE GENERATOR SCRIPT
"""

from sans.models.BaseComponent import BaseComponent
from sans.models.sans_extension.c_models import CStarPolymer

def create_StarPolymer():
    """
       Create a model instance
    """
    obj = StarPolymer()
    # CStarPolymer.__init__(obj) is called by
    # the StarPolymer constructor
    return obj

class StarPolymer(CStarPolymer, BaseComponent):
    """ 
    Class that evaluates a StarPolymer model. 
    This file was auto-generated from src\sans\models\include\starpolymer.h.
    Refer to that file and the structure it contains
    for details of the model.
    List of default parameters:
         arms            = 3.0 
         R2              = 100.0 [A]
         scale           = 1.0 
         background      = 0.0 [1/cm]

    """
        
    def __init__(self, multfactor=1):
        """ Initialization """
        self.__dict__ = {}
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        #apply(CStarPolymer.__init__, (self,)) 

        CStarPolymer.__init__(self)
        self.is_multifunc = False
		        
        ## Name of the model
        self.name = "StarPolymer"
        ## Model description
        self.description = """
          Scattering model class for 'Star polymer with Gaussian statistics'
		with
		P(q) = 2/{fv^2} * (v - (1-exp(-v)) + {f-1}/2 * (1-exp(-v))^2)
		where
		- v = u^2f/(3f-2)
		- u = <R_g^2>q^2, where <R_g^2> is the ensemble average radius of
		giration squared of an arm
		- f is the number of arms on the star
        """
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['arms'] = ['', None, None]
        self.details['R2'] = ['[A]', None, None]
        self.details['scale'] = ['', None, None]
        self.details['background'] = ['[1/cm]', None, None]

        ## fittable parameters
        self.fixed = []
        
        ## non-fittable parameters
        self.non_fittable = []
        
        ## parameters with orientation
        self.orientation_params = []

        ## parameters with magnetism
        self.magnetic_params = []

        self.category = None
        self.multiplicity_info = None
        
    def __setstate__(self, state):
        """
        restore the state of a model from pickle
        """
        self.__dict__, self.params, self.dispersion = state
        
    def __reduce_ex__(self, proto):
        """
        Overwrite the __reduce_ex__ of PyTypeObject *type call in the init of 
        c model.
        """
        state = (self.__dict__, self.params, self.dispersion)
        return (create_StarPolymer, tuple(), state, None, None)
        
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(StarPolymer())   
       	
    def run(self, x=0.0):
        """ 
        Evaluate the model
        
        :param x: input q, or [q,phi]
        
        :return: scattering function P(q)
        
        """
        return CStarPolymer.run(self, x)
   
    def runXY(self, x=0.0):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q, or [qx, qy]
        
        :return: scattering function P(q)
        
        """
        return CStarPolymer.runXY(self, x)
        
    def evalDistribution(self, x):
        """ 
        Evaluate the model in cartesian coordinates
        
        :param x: input q[], or [qx[], qy[]]
        
        :return: scattering function P(q[])
        
        """
        return CStarPolymer.evalDistribution(self, x)
        
    def calculate_ER(self):
        """ 
        Calculate the effective radius for P(q)*S(q)
        
        :return: the value of the effective radius
        
        """       
        return CStarPolymer.calculate_ER(self)
        
    def calculate_VR(self):
        """ 
        Calculate the volf ratio for P(q)*S(q)
        
        :return: the value of the volf ratio
        
        """       
        return CStarPolymer.calculate_VR(self)
              
    def set_dispersion(self, parameter, dispersion):
        """
        Set the dispersion object for a model parameter
        
        :param parameter: name of the parameter [string]
        :param dispersion: dispersion object of type DispersionModel
        
        """
        return CStarPolymer.set_dispersion(self,
               parameter, dispersion.cdisp)
        
   
# End of file

